/*************************************************
 *  Copyright (C) 1999-2008 by  Zilog, Inc.
 *  All Rights Reserved
 *************************************************/

#include <stddef.h>
#include <format.h>

#ifndef _PTR_
#define _PTR_ *
#endif


/*************************************************
*
* _u_ltoa - convert a long to a number of base rad
*
* Inputs:
*	lng - long to be converted
*	str - target char array
*	rad - radix
*	fmt - format structure with conversion info
*
* Returns:
*	next address past the end of the string
*
*************************************************/

#ifndef _MULTI_THREAD
void _u_ltoa(long val)
#else
char _mt_ltoa(long val,char* __print_buff,struct fmt_type* print_fmt)
#define __print_fmt (*print_fmt)
#endif
{
  unsigned char a;
  unsigned char t;
  unsigned char sign;
  char _PTR_ s = __print_buff;
  unsigned char nonzero;
  unsigned long uval;
  char rad;
#ifdef _MULTI_THREAD
  char __print_leading_char = '\0';
#endif

  sign = val < 0;
  uval = sign ? -val : val;
  nonzero = (uval > 0);

  if (__print_fmt.precision < 0)
    __print_fmt.precision = 1;
 
  if (__print_fmt.field_width < 0)
  {
    __print_fmt.flags |= FMT_FLAG_MINUS;
    __print_fmt.field_width = -__print_fmt.field_width;
  }

  a = 'A';
  switch (__print_fmt.type)
  {
  default:
  case 'd':
  case 'i':
  case 'u':
    rad = 10;
    break;
  case 'o':
    rad = 0;
    break;
  case 'x':
  case 'p':
    if ( !(__print_fmt.flags&FMT_FLAG_UPPERCASE))
      a = 'a';
    // Fall into next case
  case 'X':
    rad = 16;
    break;
  }
  do
    {	/* generate digits in reverse order */
    t = uval % rad;
    if (t > 9)
      *(s++) = (t - 10) + a;
    else
      *(s++) = t + '0';
    }  while (uval /= rad);

  while (s - __print_buff < (unsigned char) __print_fmt.precision) {
    *(s++) = '0';
  }
  if (__print_fmt.flags & FMT_FLAG_POUND) {
    if (nonzero) {
      if (rad == 16) {
 	*(s++) = 'X';
	*(s++) = '0';
      }
      else if (rad == 116) {
	*(s++) = 'x';
	*(s++) = '0';
      }
      else if (rad == 8) {
	*(s++) = '0';
      }
    }
  }

  if (sign)
    a = '-';
  else if (__print_fmt.flags & FMT_FLAG_PLUS)
    a = '+';
  else if (__print_fmt.flags & FMT_FLAG_SPACE)
    a = ' ';
  else
    a = '\0';

  // ISO Spec:  If a precision is specified  the zero flag is ignored
  if (__print_fmt.precision )
    __print_fmt.flags &= ~FMT_FLAG_PAD_ZERO;

  if (   __print_fmt.flags & FMT_FLAG_PAD_ZERO)
    __print_leading_char = a;
  else if (a)
    *(s++) = a;

  *s = '\0';

  _u_reverse(__print_buff);
#ifndef _MULTI_THREAD
  __print_send();
#else
  return __print_leading_char;
#endif
}

